/******************************************
 * [d7DevServer]
 * Copyright (c) 2024 Kougen.Sai
 * Released under the MIT license
 * https://opensource.org/licenses/mit-license.php 
 ******************************************/
package com.cycau.d7server.controller;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

@RestController
public class MockController {
    @Autowired
    private MockData mockData;

    private static final Logger log = LoggerFactory.getLogger("DDDDDDD");

    @RequestMapping("/mock/api/**")
    public ResponseEntity<Object> get(HttpServletRequest request) throws IOException {
    	Map<String, Object> requestData = collectRequestData(request);
    	try {
			return mockData.getResponse(request.getMethod(), request.getRequestURI().substring(9), requestData);
		} catch (Exception e) {
			log.error("Something wrong. {}", e);
	        return new ResponseEntity<>("Something wrong.", HttpStatus.FORBIDDEN);
		}
    }

    private Map<String, Object> collectRequestData(HttpServletRequest req) throws IOException {
        Map<String, Object> request = new HashMap<>();
        Map<String, Object> header = new HashMap<>();
        Map<String, Object> query = new HashMap<>();

        Enumeration<String> headerNames = req.getHeaderNames();
        while (headerNames.hasMoreElements()) {
            String name = headerNames.nextElement();
            header.put(name, req.getHeader(name));
        }
        // query + formData
        Enumeration<String> queryNames = req.getParameterNames();
        while (queryNames.hasMoreElements()) {
            String name = queryNames.nextElement();
            query.put(name, req.getParameter(name));
        }
        if (((String)header.getOrDefault("Content-Type", "")).contains("json")) {
            ObjectMapper mapper = new ObjectMapper();
            try {
                Map<String, Object> data = mapper.readValue(req.getInputStream(), new TypeReference<Map<String, Object>>(){});
                request.put("data", data);
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        } else {
        	InputStreamReader stream = new InputStreamReader(req.getInputStream());
        	BufferedReader reader = new BufferedReader(stream);
        	String line;
        	StringBuilder result = new StringBuilder();
        	while((line = reader.readLine()) != null) {
        	    result.append(line);
        	}
        	String data = result.toString();
        	request.put("data", data.isEmpty() ? null : data);
        	reader.close();
        	stream.close();
        }

        request.put("header", header.size() < 1 ? null : header);
        request.put("query", query.size() < 1 ? null : query);
        return request;
    }
}